@@ -10,6 +10,7 @@ import os.path
 
 from pypower import idx_bus, idx_brch, idx_gen
 from pypower.api import ppoption, runpf
+from xlrd.biffh import XLRDError
 import numpy
 import xlrd
 
@@ -32,7 +33,15 @@ BUS_PQ_FACTOR = power_factor * 1e6  # from MW to W
 BRANCH_PQ_FACTOR = power_factor * 1e6  # from MW to W
 
 
-def load_case(path, grid_idx):
+DEFAULT_SHEETS = {
+    'bus': 'Nodes',
+    'branch': 'Lines',
+    'branch_types': 'Line types',
+    'trafo_types': 'Transformer types',
+}
+
+
+def load_case(path, grid_idx, sheetnames):
     """Load the case from *path* and create a PYPOWER case and an entity map.
     """
     loaders = {
@@ -48,8 +57,9 @@ def load_case(path, grid_idx):
     entity_map = UniqueKeyDict()
 
     raw_case = loader.open(path)
-    buses = _get_buses(loader, raw_case, entity_map, grid_idx)
-    branches = _get_branches(loader, raw_case, entity_map, grid_idx)
+    buses = _get_buses(loader, raw_case, entity_map, grid_idx, sheetnames)
+    branches = _get_branches(loader, raw_case, entity_map, grid_idx,
+                             sheetnames)
     base_mva = loader.base_mva(raw_case, buses)
 
     ppc = _make_ppc(base_mva, buses, branches)
@@ -160,9 +170,10 @@ def case_for_eid(eid, case):
     return case[int(idx)]
 
 
-def _get_buses(loader, raw_case, entity_map, grid_idx):
+def _get_buses(loader, raw_case, entity_map, grid_idx, sheetnames):
     buses = []
-    for idx, (bid, btype, base_kv) in enumerate(loader.buses(raw_case)):
+    for idx, (bid, btype, base_kv) in enumerate(loader.buses(raw_case,
+                                                             sheetnames)):
         eid = make_eid(bid, grid_idx)
         buses.append((idx, btype, base_kv))
         etype = 'RefBus' if btype == 'REF' else 'PQBus'
@@ -176,9 +187,10 @@ def _get_buses(loader, raw_case, entity_map, grid_idx):
     return buses
 
 
-def _get_branches(loader, raw_case, entity_map, grid_idx):
+def _get_branches(loader, raw_case, entity_map, grid_idx, sheetnames):
     branches = []
-    for idx, branch in enumerate(loader.branches(raw_case, entity_map)):
+    for idx, branch in enumerate(loader.branches(raw_case, entity_map,
+                                                 sheetnames)):
         is_trafo, bid, fbus, tbus, length, bdata, online, tap_turn = branch
         eid = make_eid(bid, grid_idx)
         fbus = make_eid(fbus, grid_idx)
@@ -271,11 +283,11 @@ class JSON:
     def open(path):
         return json.load(open(path))
 
-    def buses(raw_case):
+    def buses(raw_case, sheetnames):
         for bus_id, bus_type, base_kv in raw_case['bus']:
             yield (bus_id, bus_type, base_kv)
 
-    def branches(raw_case, entity_map):
+    def branches(raw_case, entity_map, sheetnames):
         if 'base_mva' in raw_case:
             # Old format
             for tid, fbus, tbus, Sr, Uk, Pk, Imaxp, Imaxs in raw_case['trafo']:
@@ -297,12 +309,25 @@ class JSON:
 
         else:
             # New format
+            # Get trafo DB
+            data = raw_case.get('trafo_types', {}).items()
+            trafos = dict(rdb.transformers)
+            for n, d in data:
+                # Convert transformer tap levels from str to int:
+                d[-1] = {int(k): v for k, v in d[-1].items()}
+                trafos[n] = rdb.Transformer(*d)
+
+            # Get line DB
+            data = raw_case.get('branch_types', {}).items()
+            lines = dict(rdb.lines)
+            lines.update((n, rdb.Line(*d)) for n, d in data)
+
             for tid, fbus, tbus, ttype, online, tap in raw_case['trafo']:
-                trafo = rdb.transformers[ttype]
+                trafo = trafos[ttype]
                 yield (True, tid, fbus, tbus, 1, trafo, int(online), tap)
 
             for bid, fbus, tbus, btype, l, online in raw_case['branch']:
-                line = rdb.lines[btype]
+                line = lines[btype]
                 yield (False, bid, fbus, tbus, l, line, int(online), 0)
 
     def base_mva(raw_case, buses):
@@ -326,33 +351,56 @@ class Excel:
             Excel.cache[path] = wb
             return wb
 
-    def buses(wb):
-        sheet = wb.sheet_by_index(0)
-        for i in range(1, sheet.nrows):
-            if str(sheet.cell_value(i, 0)).startswith('#'):
-                continue
-
-            bus_id, bus_type, base_kv = sheet.row_values(i)[:3]
+    def buses(wb, sheetnames):
+        sheet = Excel._sheet(wb, 'bus', sheetnames)
+        for bus_id, bus_type, base_kv in Excel._iter(sheet, 3):
             if type(bus_id) is float:
                 bus_id = str(int(bus_id))
             yield (bus_id, bus_type, base_kv)
 
-    def branches(wb, entity_map):
-        sheet = wb.sheet_by_index(1)
-        for i in range(1, sheet.nrows):
-            if str(sheet.cell_value(i, 0)).startswith('#'):
-                continue
+    def branches(wb, entity_map, sheetnames):
+        # Get trafo dB
+        try:
+            sheet = Excel._sheet(wb, 'trafo_types', sheetnames)
+            data = Excel._iter(sheet, len(rdb.Transformer._fields) + 1)
+            # Parse transformer taps:
+            data = (d[:-1] + [eval(d[-1])] for d in data)
+        except XLRDError:
+            data = []
+        trafos = dict(rdb.transformers)
+        trafos.update((n, rdb.Transformer(*d)) for n, *d in data)
 
-            bid, fbus, tbus, btype, l, online, tap = sheet.row_values(i)[:7]
+        # Get line DB
+        try:
+            sheet = Excel._sheet(wb, 'branch_types', sheetnames)
+            data = Excel._iter(sheet, len(rdb.Line._fields) + 1)
+        except XLRDError:
+            data = []
+        data = list(data)
+        lines = dict(rdb.lines)
+        lines.update((n, rdb.Line(*d)) for n, *d in data)
+
+        sheet = Excel._sheet(wb, 'branch', sheetnames)
+        for bid, fbus, tbus, btype, l, online, typ in Excel._iter(sheet, 7):
             if type(bid) is float:
                 bid = str(int(bid))
             try:
-                info = rdb.transformers[btype]
+                info = trafos[btype]
                 is_trafo = True
             except KeyError:
-                info = rdb.lines[btype]
+                info = lines[btype]
                 is_trafo = False
             yield (is_trafo, bid, fbus, tbus, l, info, online, 0)
 
     def base_mva(raw_case, buses):
         return rdb.base_mva.get(buses[0][BUS_BASE_KV], 1)
+
+    def _iter(sheet, ncols):
+        for i in range(1, sheet.nrows):
+            if str(sheet.cell_value(i, 0)).startswith('#'):
+                continue
+            yield sheet.row_values(i)[:ncols]
+
+    def _sheet(wb, name, sheetnames):
+        name = sheetnames.get(name, DEFAULT_SHEETS[name])
+        return wb.sheet_by_name(name)
