@@ -19,6 +19,7 @@ meta = {
             'public': True,
             'params': [
                 'gridfile',  # Name of the file containing the grid topology.
+                'sheetnames',  # Mapping of Excel sheet names, optional.
             ],
             'attrs': [],
         },
@@ -31,7 +32,7 @@ meta = {
                 'Vl',  # Nominal bus voltage [V]
                 'Vm',  # Voltage magnitude [V]
                 'Va',  # Voltage angle [deg]
-            ]
+            ],
         },
         'PQBus': {
             'public': False,
@@ -42,7 +43,7 @@ meta = {
                 'Vl',  # Nominal bus voltage [V]
                 'Vm',  # Voltage magnitude [V]
                 'Va',  # Voltage angle [deg]
-            ]
+            ],
         },
         'Transformer': {
             'public': False,
@@ -60,7 +61,7 @@ meta = {
                 'U_s',       # Nominal secondary voltage [V]
                 'taps',      # Dict. of possible tap turns and their values
                 'tap_turn',  # Currently active tap turn
-            ]
+            ],
         },
         'Branch': {
             'public': False,
@@ -79,7 +80,7 @@ meta = {
                 'X_per_km',  # Reactance per unit length [Ω/km]
                 'C_per_km',  # Capactity per unit length [F/km]
                 'online',    # Boolean flag (True|False)
-            ]
+            ],
         },
     },
 }
@@ -113,16 +114,19 @@ class PyPower(mosaik_api.Simulator):
 
         return self.meta
 
-    def create(self, num, modelname, gridfile):
+    def create(self, num, modelname, gridfile, sheetnames=None):
         if modelname != 'Grid':
             raise ValueError('Unknown model: "%s"' % modelname)
         if not os.path.isfile(gridfile):
             raise ValueError('File "%s" does not exist!' % gridfile)
 
+        if not sheetnames:
+            sheetnames = {}
+
         grids = []
         for i in range(num):
             grid_idx = len(self._ppcs)
-            ppc, entities = model.load_case(gridfile, grid_idx)
+            ppc, entities = model.load_case(gridfile, grid_idx, sheetnames)
             self._ppcs.append(ppc)
 
             children = []
